
-----
-- tbl.until filter                              Copyright (c) 2006 Asko Kauppi
--
-- Allows the use of reserved words ("repeat", "until", "nil", "function", ...)
-- as table indices, for concistency, less syntax surprises and general Good Will usage.
--
-- Exceptions are the words "nil", "do", "function", "false" and "true", after '{' or ','
-- tokens, since these are valid values in tables, or parameter lists.
--
-- Usage:
--      lua -lftools -fdotuntil ...
--
-- Requires:    Lua 5.1, with token filter patch applied
--              ftools.lua
--
-- Note: 
--      We're glad to be able to simply change the reserved words into corresponding
--      "<name>" entries in this filter. This works, and it's simple.
-----

if type( rawget(_G,"FILTER") )~="table" then
    error "Requires ftools.lua to be loaded"
end

-----
-- 1: any reserved word after these are converted
-- 2: reserved words other than exceptions are converted
--
local match= { ['.']=1, [':']=1, ['{']=2, [',']=2 }
local class2_exceptions= { ["false"]=1, ["true"]=1, ["do"]=1, ["function"]=1, ["nil"]=1 }

-- sugar: "'.'|':'|'{'|',' reserved" --> "'.'|':'|'{'|',' <name>"
--
local function DOTUNTIL(p)
    local class
    repeat
        class= match[ p:read() ]
    until class

    local tk= p:read()
    -- avoid use of 'string.find' for performance reasons (paranoid? ;)
    local c= string.sub(tk,1,1)
    if c>='a' and c<='z' then   -- all reserved words
        if class==1 or not class2_exceptions[tk] then
            p:replace( 1, "<name>",tk )
        end
    end
end

FILTER.link( DOTUNTIL )
