
-----
-- SELECT token filter                                  Copyright (c) 2006 Asko Kauppi
--
-- Simplifies the 'select' usage a bit, making it look less like a function
--
--      #...        -> select('#',...)  = number of values in ...
--      ...[n]      -> (select(n,...))  = value #n of ...
--
-- 'select(n,...)' is still used explicitly when wanting all values from 'n' onwards.
--
-- Usage:
--      lua -lftools -fselect ...
--
-- Requires:    Lua 5.1, with token filter patch applied
--              ftools.lua
-----

if type( rawget(_G,"FILTER") )~="table" then
    error "Requires ftools.lua to be loaded"
end

local NAME="<name>"
local STRING="<string>"

-----
-- tk_str= read_hashooo( p )
--
-- Like p:read() but handled "#..." expansion; no token is returned for that.
--
local function read_hashooo(p)
    local tk= p:read()
    if tk=='#' then
        if p:peek()=="..." then
            p:read()
            p:replace( 2, NAME,"select", '(', STRING,'#', ',', "...", ')' )
            return p:read()     -- next token (cannot be "#...#")
        end
    end
    return tk
end

--
local function SELECT(p)
    -- Do the replacement of "#..." while chacing other "..."
    --
    repeat tk=read_hashooo(p) until tk=="..."

    if p:read()=='[' then
        --
        -- Note: surrounding by () is important, to cut values into just one (or zero)
        p:replace( 2, '(', NAME,"select", '(' )
        
        -- Prepare to work even with "...[t[x]]" (count the brackets)
        --
        local brackets=1
        while brackets>0 do
            tk= read_hashooo(p)     -- allows "...[#...]"
            if tk=='[' then
                brackets= brackets+1
            elseif tk==']' then
                brackets= brackets-1
            end
        end
        p:replace( 1, ',', "...", ')', ')' )
    end
end

FILTER.link( SELECT )
